"""Graph data models for LangGraph Studio.

Re-exports from castella.graph for backwards compatibility.
"""

from castella.graph import (
    GraphModel,
    NodeModel,
    NodeType,
    EdgeModel,
    EdgeType,
)

__all__ = [
    "GraphModel",
    "NodeModel",
    "NodeType",
    "EdgeModel",
    "EdgeType",
]
